﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Text;
using System.Configuration;
using System.Net;
using System.IO;
using Newtonsoft.Json.Linq;
using System.Text.RegularExpressions;
using System.Data;
using System.Data.OleDb;
using Rss;

namespace Geekees.App.SinaFeed.Lib
{
    public class SinaFeedManager
    {
        #region Configuration Properties

        protected string SinaBaseServiceUrl
        {
            get
            {
                object o = ConfigurationManager.AppSettings["SinaBaseServiceUrl"];
                return o == null ? "http://v.t.sina.com.cn/widget/getusermblog.php?uid={0}" : (string)o;
            }
        }

        protected string SinaFeedDBConnectionString
        {
            get
            {
                object o = ConfigurationManager.AppSettings["SinaFeedDBConnectionString"];

                if (o == null)
                {
                    string path = string.Empty;

                    if (HttpContext.Current != null)
                    {
                        path = HttpContext.Current.Server.MapPath("~/");
                    }
                    else
                    {
                        path = AppDomain.CurrentDomain.BaseDirectory;

                    }

                    string connStr = string.Format("Provider=Microsoft.Jet.OLEDB.4.0;Data source={0}db\\SinaFeed.mdb", path);

                    return connStr;
                }

                return (string)o;
            }
        }

        protected bool UseProxy
        {
            get
            {
                object o = ConfigurationManager.AppSettings["UseProxy"];
                return o == null ? false : (bool.Parse((string)o));
            }
        }

        protected string ProxyAddress
        {
            get
            {
                object o = ConfigurationManager.AppSettings["ProxyAddress"];
                return o == null ? "" : (string)o;
            }
        }

        protected int ProxyPort
        {
            get
            {
                object o = ConfigurationManager.AppSettings["ProxyPort"];
                return o == null ? 8080 : int.Parse((string)o);
            }
        }

        protected string ProxyDomain
        {
            get
            {
                object o = ConfigurationManager.AppSettings["ProxyDomain"];
                return o == null ? "asnet" : (string)o;
            }
        }

        protected string ProxyUserName
        {
            get
            {
                object o = ConfigurationManager.AppSettings["ProxyUserName"];
                return o == null ? "wjin" : (string)o;
            }
        }

        protected string ProxyUserPassword
        {
            get
            {
                object o = ConfigurationManager.AppSettings["ProxyUserPassword"];
                return o == null ? "Password3!" : (string)o;
            }
        }

        protected string SinaServiceUrl
        {
            get
            {
                return string.Format(this.SinaBaseServiceUrl, this.SinaUserId);
            }
        }

        #region RSS related

        protected string RssChannelUrl
        {
            get
            {
                object o = ConfigurationManager.AppSettings["RssChannelUrl"];
                return o == null ? "http://t.sina.com.cn/" + SinaUserId : (string)o;
            }
        }

        protected string RssBaseGuid
        {
            get
            {
                object o = ConfigurationManager.AppSettings["RssBaseGuid"];
                return o == null ? "http://t.sina.com.cn/" + SinaUserId + "/" : (string)o;
            }
        }

        protected string RssTitle
        {
            get
            {
                object o = ConfigurationManager.AppSettings["RssTitle"];
                return o == null ? "JIN Weijie Sina Micro Blog" : (string)o;
            }
        }

        protected string RssDescription
        {
            get
            {
                object o = ConfigurationManager.AppSettings["RssDescription"];
                return o == null ? "JIN Weijie Sina Micro Blog RSS Feed." : (string)o;
            }
        }

        #region RssMaxItemCount (int)

        private int rssMaxItemCount = 100;
        /// <summary>
        /// SinaUserId
        /// </summary>
        public int RssMaxItemCount
        {
            get
            {
                return this.rssMaxItemCount;
            }
            set
            {
                this.rssMaxItemCount = value;
            }
        }

        #endregion

        #endregion

        #endregion

        #region Properties

        #region SinaUserId (string)

        private string sinaUserId = "";
        /// <summary>
        /// SinaUserId
        /// </summary>
        public string SinaUserId
        {
            get
            {
                return this.sinaUserId;
            }
            set
            {
                this.sinaUserId = value;
            }
        }

        #endregion

        #region TweetEncoding (string)

        private string tweetEncoding = "UTF-8";
        /// <summary>
        /// TweetEncoding
        /// </summary>
        public string TweetEncoding
        {
            get
            {
                return this.tweetEncoding;
            }
            set
            {
                this.tweetEncoding = value;
            }
        }

        #endregion

        #region UseOriginalImage (bool)

        private bool useOriginalImage = true;
        /// <summary>
        /// UseOriginalImage
        /// </summary>
        public bool UseOriginalImage
        {
            get
            {
                return this.useOriginalImage;
            }
            set
            {
                this.useOriginalImage = value;
            }
        }

        #endregion

        #region UseFullTitle (bool)

        private bool useFullTitle = true;
        /// <summary>
        /// UseFullTitle
        /// </summary>
        public bool UseFullTitle
        {
            get
            {
                return this.useFullTitle;
            }
            set
            {
                this.useFullTitle = value;
            }
        }

        #endregion

        #region RemoveQuotaion (bool)

        private bool removeQuotation = true;
        /// <summary>
        /// RemoveQuotaion
        /// </summary>
        public bool RemoveQuotaion
        {
            get
            {
                return this.removeQuotation;
            }
            set
            {
                this.removeQuotation = value;
            }
        }

        #endregion

        #region TitleMaxLength (int)

        private int titleMaxLength = 50;
        /// <summary>
        /// TitleMaxLength
        /// </summary>
        public int TitleMaxLength
        {
            get
            {
                return this.titleMaxLength;
            }
            set
            {
                this.titleMaxLength = value;
            }
        }

        #endregion

        #region AttachImageToEnclosure

        bool attachImageToEnclosure = true;
        public bool AttachImageToEnclosure
        {
            get
            {
                return attachImageToEnclosure;
            }
            set
            {
                attachImageToEnclosure = value;
            }
        }

        #endregion

        #region AttachImageToTitle

        bool attachImageToTitle = false;
        public bool AttachImageToTitle
        {
            get
            {
                return attachImageToTitle;
            }
            set
            {
                attachImageToTitle = value;
            }
        }

        #endregion

        #region AttachImageToDescription

        bool attachImageToDescription = false;
        public bool AttachImageToDescription
        {
            get
            {
                return attachImageToDescription;
            }
            set
            {
                attachImageToDescription = value;
            }
        }

        #endregion

		#region AttachOriginalMblogToRT

		private bool attachOriginalMblogToRT = true;

		public bool AttachOriginalMblogToRT
		{
			get
			{
				return attachImageToDescription;
			}
			set
			{
				this.attachImageToDescription = value;
			}
		}

		#endregion

		#endregion

		#region WriteRSSFeed

		public void WriteRSSFeed( Stream stream )
        {
            /*
             RssFeed feed = new RssFeed();
            feed.Channels.Add(channel);
            Response.ContentType = "text/xml";
            feed.Write(Response.OutputStream);
            Response.End(); 
            */

            RssChannel channel = GenerateChannel();

            RssFeed feed = new RssFeed(Encoding.UTF8);
            feed.Channels.Add(channel);
            feed.Write(stream);
        }

        public void WriteRSSFeed(string fileName)
        {
            RssChannel channel = GenerateChannel();

            RssFeed feed = new RssFeed(Encoding.UTF8);
            feed.Channels.Add(channel);
            feed.Write(fileName);
        }

        #endregion

        #region GenerateChannel

        protected RssChannel GenerateChannel()
        {
            RssChannel channel = new RssChannel();
            #region channel
            channel.Title = this.RssTitle;
            channel.Description = this.RssDescription;
            channel.LastBuildDate = DateTime.Now;
            channel.Link = new Uri(RssChannelUrl);
            channel.Generator = "http://www.jinweijie.com";
            channel.Copyright = "@JIN Weijie";
            channel.Docs = "http://www.jinweijie.com";
            #endregion

            #region items

            DataTable dtMblogs = GetMblogs();
            foreach (DataRow dr in dtMblogs.Rows)
            {
                RssItem item = new RssItem();

                #region image

                string imgUrl = string.Empty;
                if (dr["Pic"] != null && dr["Pic"].ToString().Length > 0)
                {
                    string raw = dr["Pic"].ToString();
                    if (this.UseOriginalImage)
                        imgUrl = raw.Replace("thumbnail", "large");
                    else
                        imgUrl = raw;
                }

                #endregion

				int mId = (int)dr["b.Id"];
				int contentId = (int)dr["c.Id"];
                string text = string.Empty;

                DataTable dtRt = GetRtByMblogId(mId);
				if( dtRt.Rows.Count > 0 )
				{
					text = dtRt.Rows[0]["Rtreason"].ToString();
					if( this.AttachOriginalMblogToRT )
					{
						string rtNick = string.Empty;
						rtNick = GetRtUserName( contentId );
						if( string.IsNullOrEmpty( rtNick ) )
							rtNick = dtRt.Rows[0]["Rtnick"].ToString();

						text += ( " //@" + rtNick + ":" + dr["CText"].ToString() );
					}
				}
				else
					text = dr["CText"].ToString();

                #region get content
                string title = string.Empty;
                if( text.Length > this.TitleMaxLength && !this.UseFullTitle )
                    title = text.Substring( 0, this.TitleMaxLength ) + "...";
                else
                    title = text;
                #endregion

                item.Title = title;
                item.Description = text;

                #region item image 

                if (imgUrl.Length > 0)
                {
                    if (this.AttachImageToEnclosure)
                    {
                        RssEnclosure re = new RssEnclosure();
                        re.Url = new Uri(imgUrl);
                        re.Type = this.GetMIMEType(imgUrl);
                        re.Length = 1;
                        item.Enclosure = re;
                    }

                    if( this.AttachImageToTitle )
                        item.Title += (" " + imgUrl);


                    if (this.AttachImageToDescription)
                        item.Description += (" " + imgUrl);
                }
                #endregion
                #region pubDate
                int ts= (int)dr["Time"];
                DateTime dt = new DateTime(1970, 1, 1, 0, 0, 0, 0);
                dt = dt.AddSeconds(ts);
                item.PubDate = dt;
                #endregion
                string url = this.RssBaseGuid + dr["b.Mblogid"].ToString().Replace("\"","");
                #region guid
                RssGuid rg = new RssGuid();
                rg.Name = url;
                item.Guid = rg;
                item.Link = new Uri(url);
                #endregion

                channel.Items.Add(item);
            }

            #endregion

            return channel;
        }

        #endregion

        #region GetMIMEType

        protected string GetMIMEType( string url )
        {
            string s = url.ToLower();
            if (s.EndsWith("jpe") || s.EndsWith("jpg") || s.EndsWith("jpeg"))
                return "image/jpeg";
            else if (s.EndsWith("gif"))
                return "image/gif";
            else if (s.EndsWith("png"))
                return "image/png";
            else if (s.EndsWith("bm;"))
                return "image/bmp";
            else
                return "image/jpeg";
        }

        #endregion

        #region GetMblogs
        
        protected DataTable GetMblogs()
        {
            string sql = string.Format( "SELECT TOP {0} * FROM Mblog b INNER JOIN MblogContent c ON b.Id=c.MblogId WHERE Uid=@Uid ORDER BY b.[Time] DESC", this.RssMaxItemCount );

            return OleDbHelper.ExecuteDataset(this.SinaFeedDBConnectionString, CommandType.Text, sql,new OleDbParameter("@Uid", this.SinaUserId)).Tables[0];
        }

		public string GetRtUserName( int contentId )
		{
			string sql = string.Format( "SELECT * FROM MblogAt WHERE ContentId=@ContentId ORDER BY [Id] DESC" );

			DataTable dt = OleDbHelper.ExecuteDataset( this.SinaFeedDBConnectionString, CommandType.Text, sql, new OleDbParameter( "@ContentId", contentId ) ).Tables[0];

			if( dt.Rows.Count > 0 )
				return dt.Rows[0]["AtUserKey"].ToString();
			else
				return null;
		}
        
        #endregion

        #region GetSinaTweets

        public void GetSinaTweets()
        {
            string json = GetServerJson(this.SinaServiceUrl);

            JObject o = JObject.Parse(json);

            CreateLog((int)o["errno"], (string)o["errmsg"], (int)o["count"], DateTime.Now);

            JArray result = (JArray)o["result"];

            foreach (JObject a in result)
            {
                string mblogid = a["mblogid"].ToString();
                string uid = a["uid"].ToString();
                int time = (int)a["time"];
                string strtime = a["strtime"].ToString();
                string source = Convert.ToString(a["source"]);
                int rtnum = (int)a["rtnum"];
                int cmtnum = (int)a["cmtnum"];

                if (!IsMblogExists(mblogid))
                {
                    #region mblog
                    int newMblogId = StoreMblog(mblogid, uid, time, strtime, source, rtnum, cmtnum, DateTime.Now);
                    #endregion

                    #region mblog content

                    JObject content = (JObject)a["content"];
                    string text = content["text"].ToString();
                    string pic = string.Empty;
                    #region pic
                    Object opic = content["pic"];
                    if (opic.GetType().Name == "JObject")
                    {
                        string regexstr = @"http://.+";
                        var regex = new Regex(regexstr);

                        MatchCollection matchresults = regex.Matches(opic.ToString());

                        if (matchresults.Count > 0)
                        {
                            pic = matchresults[0].ToString().Replace("\"", "").Replace("\r","");
                        }
                    }
                    #endregion

                    int newMblogContentId = StoreMblogContent(newMblogId, text, pic);

                    #endregion

                    #region at users
                    Object atUsers = content["atUsers"];

                    if (atUsers.GetType().Name == "JObject")
                    {
                        JObject oAtUsers = ((JObject)atUsers);

                        foreach (var item in oAtUsers)
                        {
                            string k = item.Key;
                            string v = item.Value.ToString();

                            StoreMblogAt(newMblogContentId, k, v);
                        }
                    }
                    #endregion

                    #region rt

                    Object rtall = a["rt"];
                    if (rtall != null)
                    {
                        if (rtall.GetType().Name == "JObject")
                        {
                            var joRtall = rtall as JObject;
                            if (joRtall != null)
                            {
                                int rootuid = (int)joRtall["rootuid"];
                                string rootid = (string)joRtall["rootid"];
                                int fromuid = (int)joRtall["fromuid"];
                                string rtnick = (string)joRtall["rtnick"];
                                string rturl = (string)joRtall["rturl"];
                                string rtreason = (string)joRtall["rtreason"];

                                int rootrtnum= 0;
                                if( ((JValue)joRtall["rootrtnum"]).Type != JTokenType.Integer )
                                    int.TryParse((string)joRtall["rootrtnum"], out rootrtnum);
                                else
                                    rootrtnum = (int)joRtall["rootrtnum"];

                                StoreRt(rootuid, rootid, fromuid, rtnick, rturl, rtreason, rootrtnum, newMblogId);
                            }

                        }
                    }
                    #endregion
                }
            }

        }

        #endregion

        #region GetServerJson

        protected string GetServerJson(string url)
        {
            HttpWebRequest req = (HttpWebRequest)WebRequest.Create(url);
            req.Timeout = 15000;
            req.KeepAlive = true;
            req.AllowWriteStreamBuffering = false;
            req.AllowAutoRedirect = false;

            if (UseProxy)
            {
                WebProxy wp = new WebProxy(this.ProxyAddress, this.ProxyPort);
                wp.Credentials = new NetworkCredential(this.ProxyUserName, this.ProxyUserPassword, this.ProxyDomain);
                req.Proxy = wp;
            }
            try
            {
                HttpWebResponse response = (HttpWebResponse)req.GetResponse();
                Encoding encode = Encoding.GetEncoding("UTF-8");
                Stream stream = response.GetResponseStream();

                var reader = new StreamReader(stream, encode);
                string responseFromServer = reader.ReadToEnd();

                reader.Close();
                stream.Close();
                response.Close();
                return responseFromServer;
            }
            catch (TimeoutException te)
            {
                //log but return correct file

                return "";
            }
            catch (Exception e)
            {
                //Response.ContentType = "text/xml";
                //Response.Write("出错了" + e.Message + e.StackTrace);
                //Response.End();

                //log
                throw;
            }
        }

        #endregion

        #region CreateLog

        protected void CreateLog(int errno, string errmsg, int count, DateTime dt)
        {
            string sql = "INSERT INTO Log(Errno, Errmsg, [Count], CreateTime) VALUES( @Errno, @Errmsg, @Count, @CreateTime)";
            OleDbHelper.ExecuteNonQuery(this.SinaFeedDBConnectionString
               , CommandType.Text
               , sql
               , new OleDbParameter("@Errno", errno)
               , new OleDbParameter("@Errmsg", errmsg)
               , new OleDbParameter("@Count", count)
               , new OleDbParameter("@CreateTime", dt.ToString()));
        }

        #endregion

        #region IsMblogExists

        protected bool IsMblogExists(string mblogid)
        {
            string sql = "SELECT COUNT(*) FROM Mblog WHERE Mblogid=@Mblogid";
            int i = (int)OleDbHelper.ExecuteScalar(this.SinaFeedDBConnectionString
               , CommandType.Text
               , sql
               , new OleDbParameter("@Mblogid", mblogid));

            return i > 0;
        }

        #endregion

        #region StoreMblog

        protected int StoreMblog(string mblogid, string uid, int time, string strtime, string source, int rtnum, int cmtnum, DateTime dateTime)
        {
            string sql = "INSERT INTO Mblog(Mblogid, Uid, [Time], Strtime, Source, Rtnum, Cmtnum, CreateTime) VALUES( @Mblogid, @Uid, @Time, @Strtime, @Source, @Rtnum, @Cmtnum, @CreateTime)";
            string sql2 = "Select @@IDENTITY";


            using (OleDbConnection conn = new OleDbConnection(this.SinaFeedDBConnectionString))
            {
                OleDbHelper.ExecuteNonQuery(conn
                   , CommandType.Text
                   , sql
                   , new OleDbParameter("@Mblogid", mblogid)
                   , new OleDbParameter("@Uid", uid)
                   , new OleDbParameter("@Time", time)
                   , new OleDbParameter("@Strtime", strtime)
                   , new OleDbParameter("@Source", source)
                   , new OleDbParameter("@Rtnum", rtnum)
                   , new OleDbParameter("@Cmtnum", cmtnum)
                   , new OleDbParameter("@CreateTime", dateTime.ToString())
                   );

                int id = (int)OleDbHelper.ExecuteScalar(conn, CommandType.Text, sql2);

                return id;
            }
        }

        #endregion

        #region StoreMblogContent

        protected int StoreMblogContent(int mblogId, string text, string pic)
        {
            //process text
            if (text.StartsWith("\""))
                text = text.Substring(1);

            if (text.EndsWith("\""))
                text = text.Substring(0, text.Length - 1);


            string sql = "INSERT INTO MblogContent(Ctext, Pic, MblogId) VALUES( @Ctext, @Pic, @MblogId)";
            string sql2 = "Select @@IDENTITY";

            using (OleDbConnection conn = new OleDbConnection(this.SinaFeedDBConnectionString))
            {
                OleDbHelper.ExecuteNonQuery(conn
                   , CommandType.Text
                   , sql
                   , new OleDbParameter("@Ctext", text)
                   , new OleDbParameter("@Pic", pic)
                   , new OleDbParameter("@MblogId", mblogId)
                   );

                int id = (int)OleDbHelper.ExecuteScalar(conn, CommandType.Text, sql2);

                return id;
            }
        }

        #endregion

        #region StoreMblogAt

        protected int StoreMblogAt(int newMblogContentId, string k, string v)
        {
            string sql = "INSERT INTO MblogAt(AtUserKey, AtUserValue, ContentId) VALUES( @AtUserKey, @AtUserValue, @ContentId)";
            string sql2 = "Select @@IDENTITY";

            using (OleDbConnection conn = new OleDbConnection(this.SinaFeedDBConnectionString))
            {
                OleDbHelper.ExecuteNonQuery(conn
                   , CommandType.Text
                   , sql
                   , new OleDbParameter("@AtUserKey", k)
                   , new OleDbParameter("@AtUserValue", v)
                   , new OleDbParameter("@ContentId", newMblogContentId)
                   );

                int id = (int)OleDbHelper.ExecuteScalar(conn, CommandType.Text, sql2);

                return id;
            }
        }

        #endregion

        #region StoreRt

        protected int StoreRt(int rootuid, string rootid, int fromuid, string rtnick, string rturl, string rtreason, int rootrtnum, int mblogId)
        {
            string sql = "INSERT INTO MblogRt(Rootuid, Rootid, Fromuid, Rtnick, Rturl, Rtreason, Rootrtnum, MblogId) VALUES( @Rootuid, @Rootid, @Fromuid, @Rtnick, @Rturl, @Rtreason, @Rootrtnum, @MblogId)";
            string sql2 = "Select @@IDENTITY";

            using (OleDbConnection conn = new OleDbConnection(this.SinaFeedDBConnectionString))
            {
                OleDbHelper.ExecuteNonQuery(conn
                   , CommandType.Text
                   , sql
                   , new OleDbParameter("@Rootuid", rootuid)
                   , new OleDbParameter("@Rootid", rootid)
                   , new OleDbParameter("@Fromuid", fromuid)
                   , new OleDbParameter("@Rtnick", rtnick)
                   , new OleDbParameter("@Rturl", rturl)
                   , new OleDbParameter("@Rtreason", rtreason)
                   , new OleDbParameter("@Rootrtnum", rootrtnum)
                   , new OleDbParameter("@MblogId", mblogId) );

                int id = (int)OleDbHelper.ExecuteScalar(conn, CommandType.Text, sql2);

                return id;
            }
        }

        #endregion

		#region GetRtByMblogId

		protected DataTable GetRtByMblogId( int mblogid)
        {
            string sql = string.Format("SELECT * FROM MblogRt WHERE MblogId=@MblogId");

            return OleDbHelper.ExecuteDataset(this.SinaFeedDBConnectionString, CommandType.Text, sql, new OleDbParameter("@MblogId", mblogid)).Tables[0];
		}

		#endregion

		#region CleanDB
		public void CleanDB()
        {
            string sql = "DELETE FROM Mblog";
            OleDbHelper.ExecuteNonQuery(this.SinaFeedDBConnectionString, CommandType.Text, sql);

            string sql2 = "DELETE FROM Log";
            OleDbHelper.ExecuteNonQuery(this.SinaFeedDBConnectionString, CommandType.Text, sql2);
        }
        #endregion
    }
}
